local L = AceLibrary("AceLocale-2.2"):new("EpicCountFu")
local Tablet = AceLibrary("Tablet-2.0")

local options = {
	handler = EpicCountFu,
	type = "group",
	args = {
		sep1 = {
			type = "header",
			order = 1.2,
		},
		emptyslots = {
			type = "toggle",
			order = 1.21,
			name = L["Empty Slots"],
			desc = L["Include Empty Slots"],
			get = "IsShowEmptySlots",
			set = "ToggleShowEmptySlots",
		},
		advanced = {
			type = "group",
			order = 99,
			name = L["Advanced Options"],
			desc = L["Advanced Options"],
			args = {
				smartexclude = {
					type = "toggle",
					order = 1.5,
					name = L["\"Smart\" Exclude"],
					desc = L["\"Smart\" Exclude"],
					get = "IsSmartExclude",
					set = "ToggleSmartExclude",
				}
			}
		}
	},
}

EpicCountFu = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0","AceConsole-2.0","AceDB-2.0", "FuBarPlugin-2.0", "AceDebug-2.0")

EpicCountFu.version = "$Release: 0 $"
EpicCountFu.hasIcon = "Interface\\Icons\\INV_Elemental_Primal_Mana"
EpicCountFu.clickableTooltip = true
EpicCountFu.independentProfile = true
EpicCountFu.tooltipHiddenWhenEmpty = true
EpicCountFu.debugFrame = ChatFrame5

EpicCountFu:RegisterChatCommand({"/epiccount", "/epicount"}, options)
EpicCountFu.OnMenuRequest = options

function EpicCountFu:OnClick()
	self:Update()
end

EpicCountFu:RegisterDB("EpicCountFuDB")
EpicCountFu:RegisterDefaults("profile", {
	emptyslots = false,
	optional_slots = false,
	smartexclude = true,
})

EpicCountFu._data = {}
EpicCountFu._slots = {}

local slots = {
	"HeadSlot",
	"NeckSlot",
	"ShoulderSlot",
	"BackSlot",
	"ChestSlot",
	"WristSlot",
	"HandsSlot", 
	"WaistSlot",
	"LegsSlot",
	"FeetSlot",
	"Finger0Slot",
	"Finger1Slot",
	"Trinket0Slot",
	"Trinket1Slot",
	"MainHandSlot",
	"SecondaryHandSlot",
	"RangedSlot",
}

local optional_slots = {
	"ShirtSlot",
	"TabardSlot",
}

function EpicCountFu:OnInitialize()
end

function EpicCountFu:OnEnable()
	self:RegisterEvent("UNIT_STATS", function() self:Update() end)
	self:Update()
end

function EpicCountFu:OnProfileEnable()
	self:Update()
end

function EpicCountFu:OnDataUpdate()
	self._data = {
		[6] = 0,
		[5] = 0,
		[4] = 0,
		[3] = 0,
		[2] = 0,
		[1] = 0,
		[0] = 0,
		empty = 0,
		empties = {},
		slots = {
			[6] = {},
			[5] = {},
			[4] = {},
			[3] = {},
			[2] = {},
			[1] = {},
			[0] = {},
		}
	}
	self._slots = {}
	if (self.db.profile.smartexclude) then
		self:SE_run();
	else
		self:runSlotFor(slots);
		if (self.db.profile.optional_slots) then
			self:runSlotFor(optional_slots);
		end
	end
end

function EpicCountFu:runSlotFor(slots)
	for _,slotname in pairs(slots) do
		self:count(self:process(slotname))
	end
end

function EpicCountFu:OnTextUpdate()
	self:OnDataUpdate()
	local text = "";
	for ri = 0, 6 do
		local i = 6 - ri;
		if (self._data[i] ~= 0) then
			local r, g, b, hex = GetItemQualityColor(i);
			text = text .. " " .. hex .. self._data[i] .. "|r"
		end
	end
	if (self.db.profile.emptyslots and self._data.empty ~= 0) then
		local r, g, b, hex = GetItemQualityColor(0);
		text = text .. " E:" .. hex .. self._data.empty .. "|r"
	end
	self:SetText(text);
end

function EpicCountFu:OnTooltipUpdate()
	for ri = 0, 6 do
		local i = 6 - ri;
		if (self._data[i] ~= 0) then
			local r, g, b, hex = GetItemQualityColor(i);
			local category = Tablet:AddCategory(
				"id", "rarity" .. i
			)
			local items = {}
			for _,slotname in pairs(self._data.slots[i]) do
				local slotdata = self:process(slotname);
				tinsert(items, slotdata);
			end
			sort(items,function(a,b) return a.itemName<b.itemName end);
			
			for _,slotData in ipairs(items) do
				category:AddLine(
					'text', slotData.itemName,
					'textR', r,
					'textG', g,
					'textB', b
				)
			end
		end
	end
	
	if (self.db.profile.emptyslots and self._data.empty ~= 0) then
		local r, g, b, hex = GetItemQualityColor(0);
		local category = Tablet:AddCategory(
			"id", "empty"
		)
		
		category:AddLine(
			'text', self._data.empty .. " empty slot(s)",
			'textR', r,
			'textG', g,
			'textB', b
		)
		for _,slotname in pairs(self._data.empties) do
			category:AddLine(
				'text',slotname,
				'textR', r,
				'textG', g,
				'textB', b
			)
		end
	end
end

function EpicCountFu:process(slot)
	if (self._slots[slot] == nil) then
		local itemLink = GetInventoryItemLink("player", GetInventorySlotInfo(slot) )
		if (itemLink == nil) then
			self._slots[slot] = {
				["empty"] = true,
				["slotName"] = slot,
			}
		else
			local itemName, _, itemRarity, _, _, itemType, itemSubType, _, _, _ = GetItemInfo(itemLink);
			self._slots[slot] = {
				["empty"] = false,
				["slotName"] = slot,
				["itemLink"] = itemLink,
				["itemName"] = itemName,
				["itemRarity"] = itemRarity,
				["itemType"] = itemType,
				["itemSubType"] = itemSubType,
			}
		end
	end
	return self._slots[slot];
end

function EpicCountFu:count(slotdata)
	if (slotdata.empty or slotdata.itemRarity == nil or slotdata.itemRarity < 0 or slotdata.itemRarity > 6) then
		self._data.empty = self._data.empty + 1;
		self._data.empties[self._data.empty] = slotdata.slotName;
	else
		local idx = self._data[slotdata.itemRarity] + 1;
		self._data[slotdata.itemRarity] = idx;
		self._data.slots[slotdata.itemRarity][idx] = slotdata.slotName;
	end
end

function EpicCountFu:IsShowEmptySlots()
	return self.db.profile.emptyslots
end

function EpicCountFu:ToggleShowEmptySlots()
	self.db.profile.emptyslots = not self.db.profile.emptyslots
	self:Update()
	return self.db.profile.emptyslots
end

function EpicCountFu:IsSmartExclude()
	return self.db.profile.smartexclude
end

function EpicCountFu:ToggleSmartExclude()
	self.db.profile.smartexclude = not self.db.profile.smartexclude
	self:Update()
	return self.db.profile.smartexclude
end
