local BI = AceLibrary("Babble-Inventory-2.2")

local function se_function_level(_level)
	return function (self,slot)
		local level = UnitLevel("player");
		return not slot.empty or level >= _level;
	end
end

local function se_function_levelother(_level,_other)
	return function (self,slot)
		local other = self:process(_other);
		local level = UnitLevel("player");
		return not slot.empty or (level < _level and not other.empty) or level >= _level;
	end
end

-- For Smart Exclude
local se_slot_data = {
	["HeadSlot"] = se_function_level(30),
	["NeckSlot"] = se_function_level(15),
	["ShoulderSlot"] = se_function_level(18),
	["BackSlot"] = se_function_level(10),
	["ChestSlot"] = true,
	["WristSlot"] = true,
	["HandsSlot"] = true, 
	["WaistSlot"] = true,
	["LegsSlot"] = true,
	["FeetSlot"] = true,
	
	["Finger0Slot"] = se_function_levelother(15,"Finger1Slot"),
	["Finger1Slot"] = se_function_levelother(15,"Finger0Slot"),
	
	["Trinket0Slot"] = se_function_levelother(30,"Trinket1Slot"),
	["Trinket1Slot"] = se_function_levelother(30,"Trinket0Slot"),
	["MainHandSlot"] = true,
	["SecondaryHandSlot"] = function (self,slot)
		local _, englishClass = UnitClass("player");
		local mh = self:process("MainHandSlot");
		local st = mh.itemSubType;
		if (not slot.empty) then return true; end
		if (st == BI["Staves"] or
			st == BI["Two-Handed Axes"] or
			st == BI["Two-Handed Maces"] or
			st == BI["Two-Handed Swords"]) then return false; end
		return true;
	end,
	["RangedSlot"] = function (self,slot)
		local localizedClass, englishClass = UnitClass("player");
		local level = UnitLevel("player");
		if (not slot.empty) then return true; end
		if (englishClass == "DRUID") then return level >= 50; end
		if (englishClass == "PALADIN") then return level >= 50; end
		if (englishClass == "SHAMAN") then return level >= 60; end
		return true;
	end,
	["ShirtSlot"] = function (self,slot)
		return (not slot.empty) and slot.itemRarity > 1
	end,
	["TabardSlot"] = function (self,slot)
		return (not slot.empty) and slot.itemRarity > 1
	end,
	["AmmoSlot"] = function (self,slot)
		return (not slot.empty) and slot.itemRarity > 2
	end,
}

function EpicCountFu:SE_run()
	for slotname,slotfunc in pairs(se_slot_data) do
		local slotdata = self:process(slotname);
		if ((type(slotfunc) == "function" and slotfunc(self,slotdata))
		 		or (type(slotfunc) ~= "function" and slotfunc)) then
			self:count(slotdata);
		end
	end
end